#!/usr/bin/perl -w

# WARNING: This file is machine-generated; any changes will be lost.
# Instead, change mkconfig.pl and the system description files.

# If you find a bug in this program (such as generation of incorrect
# options), please mail lloyd\@randombit.net with details.

# This file is in the public domain.

use strict;
use DirHandle;
use Getopt::Long;
use File::Spec::Functions;
use File::Copy;

my $MAJOR_VERSION = 0;
my $MINOR_VERSION = 7;
my $PATCH_VERSION = 2;

my $INCLUDE_DIR = catdir ('include');
my $SRC_DIR = 'src';
my $MOD_DIR = 'modules';
my $CHECK_DIR = 'checks';

my $BUILD_DIR = 'build';
my $BUILD_SRC_DIR = catdir ($BUILD_DIR, 'src');
my $BUILD_INC_DIR = catdir ($BUILD_DIR, 'include', 'opencl');
my $BUILD_LIB_DIR = catdir ($BUILD_DIR, 'lib');
my $BUILD_CHECK_DIR = catdir ($BUILD_DIR, 'checks');

my $CONFIG_H_FILE = catfile($INCLUDE_DIR, 'config.h');
my $VERSION_H_FILE = catfile($INCLUDE_DIR, 'version.h');
my $MAKE_FILE = 'Makefile';

my %ARCH = (
   '68020'        => 'm68k',
   '68030'        => 'm68k',
   '68040'        => 'm68k',
   '68060'        => 'm68k',
   'alpha'        => 'alpha',
   'alphaev4'     => 'alpha',
   'alphaev5'     => 'alpha',
   'alphaev56'    => 'alpha',
   'alphaev6'     => 'alpha',
   'alphaev67'    => 'alpha',
   'alphapca56'   => 'alpha',
   'hppa'         => 'hppa',
   'hppa1.0'      => 'hppa',
   'hppa1.1'      => 'hppa',
   'hppa2.0'      => 'hppa',
   'i386'         => 'ia32',
   'i486'         => 'ia32',
   'i586'         => 'ia32',
   'i686'         => 'ia32',
   'ia32'         => 'ia32',
   'ia64'         => 'ia64',
   'm68k'         => 'm68k',
   'mips'         => 'mips',
   'mips64'       => 'mips64',
   'powerpc'      => 'powerpc',
   'ppc601'       => 'powerpc',
   'ppc602'       => 'powerpc',
   'ppc603'       => 'powerpc',
   'ppc604'       => 'powerpc',
   'r10000'       => 'mips64',
   'r3000'        => 'mips',
   'r4000'        => 'mips64',
   'r4400'        => 'mips64',
   'r5000'        => 'mips64',
   'r6000'        => 'mips',
   'r8000'        => 'mips64',
   'sparc'        => 'sparc',
   'sparc32-v7'   => 'sparc',
   'sparc32-v8'   => 'sparc',
   'sparc32-v9'   => 'sparc',
   'sparc64'      => 'sparc64',
   'ultra'        => 'sparc64',
   'ultra2'       => 'sparc64',
);

my %ARCH_ALIAS = (
   '680x0'        => 'm68k',
   '68k'          => 'm68k',
   '80x86'        => 'ia32',
   'alphaaxp'     => 'alpha',
   'axp'          => 'alpha',
   'hp-pa'        => 'hppa',
   'hp-pa-risc'   => 'hppa',
   'hp-parisc'    => 'hppa',
   'ix86'         => 'ia32',
   'merced'       => 'ia64',
   'pa-risc'      => 'hppa',
   'parisc'       => 'hppa',
   'ppc'          => 'powerpc',
   'x86'          => 'ia32',
);

my %DEFAULT_SUBMODEL = (
   'alpha'        => 'alphaev4',
   'hppa'         => 'hppa1.0',
   'ia32'         => 'i386',
   'ia64'         => 'ia64',
   'm68k'         => '68020',
   'mips'         => 'r3000',
   'mips64'       => 'r4400',
   'powerpc'      => 'ppc602',
   'sparc'        => 'sparc32-v8',
   'sparc64'      => 'ultra',
);

my %SUBMODEL_ALIAS = (
   'athlon'       => 'i686',
   'cypress'      => 'sparc32-v7',
   'ev4'          => 'alphaev4',
   'ev5'          => 'alphaev5',
   'ev56'         => 'alphaev56',
   'ev6'          => 'alphaev6',
   'ev67'         => 'alphaev67',
   'hypersparc'   => 'sparc32-v8',
   'k6'           => 'i586',
   'microsparc'   => 'sparc32-v8',
   'mipsbe'       => 'mips3000',
   'mipsle'       => 'mips3000',
   'pca56'        => 'alphapca56',
   'pentium'      => 'i586',
   'pentium2'     => 'i686',
   'pentium3'     => 'i686',
   'pentium4'     => 'i686',
   'pentium_pro'  => 'i686',
   'pentiumpro'   => 'i686',
   'r10k'         => 'r10000',
   'r3k'          => 'r3000',
   'r4k'          => 'r4000',
   'r5k'          => 'r5000',
   'r6k'          => 'r6000',
   'r8k'          => 'r8000',
   'sparc-v7'     => 'sparc32-v7',
   'sparc-v8'     => 'sparc32-v8',
   'sparc-v9'     => 'sparc32-v9',
   'sparcv7'      => 'sparc32-v7',
   'sparcv8'      => 'sparc32-v8',
   'sparcv9'      => 'sparc32-v9',
   'supersparc'   => 'sparc32-v8',
);

my %OS_SUPPORTS_ARCH = (
   'aix'        => { 
      'ia64'       => undef,
      'powerpc'    => undef,
        },
   'beos'       => { 
      'ia32'       => undef,
      'powerpc'    => undef,
        },
   'cygwin'     => { 
      'ia32'       => undef,
        },
   'darwin'     => { 
      'ia32'       => undef,
      'powerpc'    => undef,
        },
   'freebsd'    => { 
      'alpha'      => undef,
      'ia32'       => undef,
        },
   'hpux'       => { 
      'hppa'       => undef,
        },
   'irix'       => { 
      'mips'       => undef,
      'mips64'     => undef,
        },
   'linux'      => { 
      'alpha'      => undef,
      'hppa'       => undef,
      'ia32'       => undef,
      'ia64'       => undef,
      'm68k'       => undef,
      'mips'       => undef,
      'mips64'     => undef,
      'powerpc'    => undef,
      'sparc'      => undef,
      'sparc64'    => undef,
        },
   'netbsd'     => { 
      'alpha'      => undef,
      'hppa'       => undef,
      'ia32'       => undef,
      'm68k'       => undef,
      'mips'       => undef,
      'mips64'     => undef,
      'powerpc'    => undef,
      'sparc'      => undef,
      'sparc64'    => undef,
        },
   'openbsd'    => { 
      'alpha'      => undef,
      'hppa'       => undef,
      'ia32'       => undef,
      'm68k'       => undef,
      'mips'       => undef,
      'powerpc'    => undef,
      'sparc'      => undef,
        },
   'solaris'    => { 
      'ia32'       => undef,
      'ia64'       => undef,
      'sparc'      => undef,
      'sparc64'    => undef,
        },
   'sunos'      => { 
      'sparc'      => undef,
        },
   'tru64'      => { 
      'alpha'      => undef,
        },
   'win32'      => { 
      'alpha'      => undef,
      'ia32'       => undef,
      'ia64'       => undef,
        },
);

my %OS_SUPPORTS_SHARED = (
   'aix'          => '1',
   'beos'         => '1',
   'cygwin'       => '0',
   'darwin'       => '1',
   'freebsd'      => '1',
   'hpux'         => '1',
   'irix'         => '1',
   'linux'        => '1',
   'netbsd'       => '0',
   'openbsd'      => '0',
   'solaris'      => '1',
   'sunos'        => '1',
   'tru64'        => '1',
   'win32'        => '0',
);

my %OS_SUPPORTS_STD = (
   'aix'        => { 
      'posix1'     => '1',
      'posix1b'    => '1',
      'pthreads'   => '1',
        },
   'beos'       => { 
      'posix1'     => '1',
      'posix1b'    => '1',
      'pthreads'   => '1',
        },
   'cygwin'     => { 
      'posix1'     => '1',
      'posix1b'    => '0',
      'pthreads'   => '0',
        },
   'darwin'     => { 
      'posix1'     => '1',
      'posix1b'    => '1',
      'pthreads'   => '1',
        },
   'freebsd'    => { 
      'posix1'     => '1',
      'posix1b'    => '1',
      'pthreads'   => '1',
        },
   'hpux'       => { 
      'posix1'     => '1',
      'posix1b'    => '1',
      'pthreads'   => '1',
        },
   'irix'       => { 
      'posix1'     => '1',
      'posix1b'    => '1',
      'pthreads'   => '1',
        },
   'linux'      => { 
      'posix1'     => '1',
      'posix1b'    => '1',
      'pthreads'   => '1',
        },
   'netbsd'     => { 
      'posix1'     => '1',
      'posix1b'    => '0',
      'pthreads'   => '0',
        },
   'openbsd'    => { 
      'posix1'     => '1',
      'posix1b'    => '0',
      'pthreads'   => '0',
        },
   'solaris'    => { 
      'posix1'     => '1',
      'posix1b'    => '1',
      'pthreads'   => '1',
        },
   'sunos'      => { 
      'posix1'     => '1',
      'posix1b'    => '0',
      'pthreads'   => '0',
        },
   'tru64'      => { 
      'posix1'     => '1',
      'posix1b'    => '1',
      'pthreads'   => '1',
        },
   'win32'      => { 
      'posix1'     => '0',
      'posix1b'    => '0',
      'pthreads'   => '0',
        },
);

my %OS_TYPE = (
   'aix'          => 'unix',
   'beos'         => 'unix',
   'cygwin'       => 'unix',
   'darwin'       => 'unix',
   'freebsd'      => 'unix',
   'hpux'         => 'unix',
   'irix'         => 'unix',
   'linux'        => 'unix',
   'netbsd'       => 'unix',
   'openbsd'      => 'unix',
   'solaris'      => 'unix',
   'sunos'        => 'unix',
   'tru64'        => 'unix',
   'win32'        => 'windows',
);

my %CC_SUPPORTS_OS = (
   'compaq'     => { 
      'linux'      => undef,
      'tru64'      => undef,
        },
   'hpcc'       => { 
      'hpux'       => undef,
        },
   'kai'        => { 
      'hpux'       => undef,
      'irix'       => undef,
      'linux'      => undef,
      'solaris'    => undef,
      'tru64'      => undef,
        },
   'mipspro'    => { 
      'irix'       => undef,
        },
   'pgi'        => { 
      'linux'      => undef,
      'solaris'    => undef,
        },
   'sunwspro'   => { 
      'solaris'    => undef,
        },
);

my %CC_SUPPORTS_ARCH = (
   'compaq'     => { 
      'alpha'      => undef,
        },
   'hpcc'       => { 
      'hppa'       => undef,
        },
   'kai'        => { 
      'alpha'      => undef,
      'hppa'       => undef,
      'ia32'       => undef,
      'mips'       => undef,
      'mips64'     => undef,
      'sparc'      => undef,
      'sparc64'    => undef,
        },
   'mipspro'    => { 
      'mips'       => undef,
      'mips64'     => undef,
        },
   'pgi'        => { 
      'ia32'       => undef,
        },
   'sunwspro'   => { 
      'ia32'       => undef,
      'sparc'      => undef,
      'sparc64'    => undef,
        },
);

my %CC_BINARY_NAME = (
   'compaq'       => 'cxx',
   'gcc'          => 'g++',
   'hpcc'         => 'CC',
   'kai'          => 'KCC',
   'mipspro'      => 'CC',
   'pgi'          => 'pgCC',
   'sunwspro'     => 'CC',
);

my %CC_OPT_FLAGS = (
   'compaq'       => '-O4 -fast',
   'gcc'          => '-O3 -fstrict-aliasing',
   'hpcc'         => '+O4 +Oinlinebudge=150',
   'kai'          => '+K3 --inline_auto_space_time=65 --abstract_pointer',
   'mipspro'      => '-O3 -OPT:alias=TYPED',
   'pgi'          => '-fast -Minline',
   'sunwspro'     => '-x04',
);

my %CC_WARN_FLAGS = (
   'compaq'       => '',
   'gcc'          => '-W -Wall',
   'hpcc'         => '+w',
   'kai'          => '',
   'mipspro'      => '-fullwarn',
   'pgi'          => '',
   'sunwspro'     => '+w',
);

my %CC_LANG_FLAGS = (
   'compaq'       => '-std ansi',
   'gcc'          => '-ansi -pedantic',
   'hpcc'         => '-ext +eh -z',
   'kai'          => '--strict -D__KAI_STRICT',
   'mipspro'      => '-ansi -LANG:ansi-for-init-scope=ON',
   'pgi'          => '',
   'sunwspro'     => '',
);

my %CC_DEBUG_FLAGS = (
   'compaq'       => '',
   'gcc'          => '-g',
   'hpcc'         => '',
   'kai'          => '',
   'mipspro'      => '',
   'pgi'          => '',
   'sunwspro'     => '',
);

my %CC_NO_DEBUG_FLAGS = (
   'compaq'       => '',
   'gcc'          => '-fomit-frame-pointer',
   'hpcc'         => '',
   'kai'          => '',
   'mipspro'      => '',
   'pgi'          => '',
   'sunwspro'     => '',
);

my %CC_MACHINE_OPT_FLAGS = (
   'compaq'     => { 
      'alpha'      => '-arch=SUBMODEL',
        },
   'gcc'        => { 
      'alpha'      => '-mcpu=SUBMODEL',
      'alphaev67'  => '-mcpu=ev6',
      'hppa'       => '-march=SUBMODEL',
      'i386'       => '-mcpu=i486',
      'ia32'       => '-march=SUBMODEL',
      'm68k'       => '-mSUBMODEL',
      'mips'       => '-mips1 -mcpu=SUBMODEL',
      'mips64'     => '-mips3 -mcpu=SUBMODEL',
      'powerpc'    => '-mcpu=powerpc',
      'r10000'     => '-mips4 -mcpu=r8000',
      'sparc'      => '-mcpu=SUBMODEL',
      'sparc64'    => '-mcpu=v9',
        },
   'hpcc'       => { 
      'hppa1.0'    => '+DAportable',
      'hppa1.1'    => '+DA1.1',
      'hppa2.0'    => '+DA2.0W',
        },
   'mipspro'    => { 
      'mips'       => '-mips1',
      'mips64'     => '-mips3',
      'r10000'     => '-mips4 -r10000',
      'r5000'      => '-mips4 -r5000',
      'r8000'      => '-mips4 -r8000',
        },
   'pgi'        => { 
      'i586'       => '-tp p5',
      'i686'       => '-tp p6',
      'ia32'       => '-tp px',
        },
   'sunwspro'   => { 
      'i386'       => '-xtarget=386',
      'i486'       => '-xtarget=486',
      'i586'       => '-xtarget=pentium',
      'i686'       => '-xtarget=pentium_pro',
      'sparc'      => '-xchip=ultra -xarch=SUBMODEL',
      'sparc64'    => '-xchip=SUBMODEL -xarch=v9',
      'sparcv9'    => '-xchip=ultra -xarch=v8',
        },
);

my %CC_MACHINE_OPT_FLAGS_RE = (
   'compaq'     => { 
      'alpha'      => 'alpha',
        },
   'gcc'        => { 
      'alpha'      => 'alpha',
      'hppa'       => 'hppa',
      'sparc'      => 'sparc32-',
        },
   'sunwspro'   => { 
      'sparc'      => 'sparc32-',
        },
);

my %CC_SO_OBJ_FLAGS = (
   'compaq'       => '',
   'gcc'          => '-fPIC',
   'hpcc'         => '+Z',
   'kai'          => '',
   'mipspro'      => '-KPIC',
   'pgi'          => '-fPIC',
   'sunwspro'     => '-KPIC',
);

my %CC_MACH_OPT_LINK_FLAGS = (
   'gcc'        => { 
      'mips'       => '-mabi=n32',
      'mips64'     => '-mabi=64',
      'sparc64'    => '-m64',
        },
   'mipspro'    => { 
      'mips'       => '-n32',
      'mips64'     => '-64',
        },
);

my %CC_SO_LINK_FLAGS = (
   'compaq'     => { 
      'default'    => '-shared -soname $(SONAME)',
        },
   'gcc'        => { 
      'default'    => '-shared -fPIC -Wl,-soname,$(SONAME)',
      'hpux'       => '-shared -fPIC -Wl,+h,$(SONAME)',
      'solaris'    => '-shared -fPIC -Wl,-h,$(SONAME) -mimpure-text',
        },
   'hpcc'       => { 
      'default'    => '-Wl,+h,$(SONAME)',
        },
   'kai'        => { 
      'default'    => '--soname $(SONAME)',
        },
   'mipspro'    => { 
      'default'    => '-shared -Wl,-soname,$(SONAME)',
        },
   'pgi'        => { 
      'linux'      => '-shared -fPIC -Wl,-soname,$(SONAME)',
      'solaris'    => '-G -fPIC -Wl,-h,$(SONAME)',
        },
   'sunwspro'   => { 
      'default'    => '-G -h$(SONAME)',
        },
);

my %REALNAME = (
   'aix'          => 'AIX',
   'alpha'        => 'DEC Alpha',
   'beos'         => 'BeOS',
   'compaq'       => 'Compaq C++',
   'cygwin'       => 'Cygwin',
   'darwin'       => 'Darwin',
   'freebsd'      => 'FreeBSD',
   'gcc'          => 'GNU C++',
   'hpcc'         => 'HP-UX C++',
   'hppa'         => 'HP-PA',
   'hpux'         => 'HP-UX',
   'ia32'         => 'IA-32',
   'ia64'         => 'IA-64',
   'irix'         => 'SGI IRIX',
   'kai'          => 'KAI C++',
   'linux'        => 'Linux',
   'm68k'         => 'Motorola 680x0',
   'mips'         => 'MIPS',
   'mips64'       => 'MIPS 64-bit',
   'mipspro'      => 'SGI MIPSPro C++',
   'netbsd'       => 'NetBSD',
   'openbsd'      => 'OpenBSD',
   'pgi'          => 'Portland Group C++',
   'posix1'       => 'POSIX.1',
   'posix1b'      => 'POSIX.1b',
   'powerpc'      => 'PowerPC',
   'pthreads'     => 'POSIX Threads',
   'solaris'      => 'Solaris',
   'sparc'        => 'Sun SPARC',
   'sparc64'      => 'Sun 64-bit SPARC',
   'sunos'        => 'SunOS',
   'sunwspro'     => 'Sun Workshop Pro C++',
   'tru64'        => 'Compaq Tru64',
   'win32'        => 'Win32',
);

my %MODULES = get_modules_list($MOD_DIR);

if($#ARGV < 0) { help(); }

my $debug = 0;
my $no_asm = 0;
my @using_mods;

GetOptions('debug' => sub { $debug = 1; $no_asm = 1; },
           'disable-asm' => \$no_asm,
           'modules=s' => \@using_mods,
           'help' => sub { help(); }
           );
@using_mods = split(/,/,join(',',@using_mods));
foreach (@using_mods) {
   die "ERROR: Module $_ isn't known (try --help)\n"
      unless(exists($MODULES{$_}));
}

if($#ARGV != 0) { help(); }

my ($cc,$os,$submodel) = split(/-/,$ARGV[0],3);
if(!defined($cc) or !defined($os) or !defined($submodel)) { help(); }

if(!defined($CC_BINARY_NAME{$cc}))
   { die "ERROR: Compiler $cc isn't known\n"; }
if(!defined($OS_SUPPORTS_ARCH{$os}) and $os ne 'generic')
   { die "ERROR: OS $os isn't known\n"; }

# Get the canonical submodel name (like r8k -> r8000)
if(defined($SUBMODEL_ALIAS{$submodel}))
   { $submodel = $SUBMODEL_ALIAS{$submodel}; }

my $arch = undef;
# Convert an arch alias to it's real name (like axp -> alpha)
if(defined($ARCH_ALIAS{$submodel}))
   { $arch = $ARCH_ALIAS{$submodel}; $submodel = $arch; }
# If it's a regular submodel type, figure out what arch it is
elsif(defined($ARCH{$submodel}))
   { $arch = $ARCH{$submodel}; }
elsif($submodel eq 'generic') { $arch = 'generic'; }
else { die "ERROR: Arch $submodel isn't known\n"; }

# If we got a generic family name as the model type
if($submodel eq $arch and $submodel ne 'generic') {
   $submodel = $DEFAULT_SUBMODEL{$arch};
   if($submodel ne $arch) {
      print "WARNING: Using $submodel as default type for family ",
            $REALNAME{$arch},"\n";
   }
}

# Does the OS support the architecture?
unless($arch eq 'generic' or $os eq 'generic' or
        exists($OS_SUPPORTS_ARCH{$os}{$arch}))
   { die "ERROR: $REALNAME{$os} doesn't run on $REALNAME{$arch}\n"; }

# Does the compiler support the architecture?
unless($arch eq 'generic' or (not exists($CC_SUPPORTS_ARCH{$cc}) or
                                  exists($CC_SUPPORTS_ARCH{$cc}{$arch})))
   { die "ERROR: $REALNAME{$cc} doesn't run on $REALNAME{$arch}\n"; }

# Does the compiler support the OS?
unless($os eq 'generic' or (not exists($CC_SUPPORTS_OS{$cc}) or
                                exists($CC_SUPPORTS_OS{$cc}{$os})))
   { die "ERROR: $REALNAME{$cc} doesn't run on $REALNAME{$os}\n"; }

my $opt_flags = $CC_OPT_FLAGS{$cc};
if(($debug == 0) and ($CC_NO_DEBUG_FLAGS{$cc}))
   { $opt_flags .= ' '.$CC_NO_DEBUG_FLAGS{$cc}; }
if($debug and ($CC_DEBUG_FLAGS{$cc}))
   { $opt_flags .= ' '.$CC_DEBUG_FLAGS{$cc}; }

my $mach_flags = "";
if(defined($CC_MACHINE_OPT_FLAGS{$cc}{$submodel}))
   { $mach_flags = $CC_MACHINE_OPT_FLAGS{$cc}{$submodel}; }
elsif(defined($CC_MACHINE_OPT_FLAGS{$cc}{$arch})) {
   $mach_flags = $CC_MACHINE_OPT_FLAGS{$cc}{$arch};
   my $processed_modelname = $submodel;
   if(defined($CC_MACHINE_OPT_FLAGS_RE{$cc}{$arch}))
      { $processed_modelname =~
           s/$CC_MACHINE_OPT_FLAGS_RE{$cc}{$arch}//; }
   $mach_flags =~ s/SUBMODEL/$processed_modelname/g;
}

my $so_link_flags = "";
if(not $OS_SUPPORTS_SHARED{$os}) { }
elsif(defined($CC_SO_LINK_FLAGS{$cc}{$os}))
   { $so_link_flags = $CC_SO_LINK_FLAGS{$cc}{$os}; }
elsif(defined($CC_SO_LINK_FLAGS{$cc}{'default'}))
   { $so_link_flags = $CC_SO_LINK_FLAGS{$cc}{'default'}; }

# These are options which must be passed every time the compiler is
# invoked (usually linking things)
my $ccopts = "";
if(defined($CC_MACH_OPT_LINK_FLAGS{$cc}{$arch}))
   { $ccopts .= ' '.$CC_MACH_OPT_LINK_FLAGS{$cc}{$arch}; }
if(defined($CC_MACH_OPT_LINK_FLAGS{$cc}{$submodel}))
   { $ccopts .= ' '.$CC_MACH_OPT_LINK_FLAGS{$cc}{$submodel}; }

check_for_conflicts(@using_mods);
my (%ignored_src, %ignored_include, %added_src, %added_include, %patched);
foreach (@using_mods) {
   load_module($_, $cc, $os, $arch, $submodel, $no_asm, %{ $MODULES{$_} });
}
# FIXME: automagically load asm_$arch_* modules, unless $no_asm is set

my %lib_src = list_dir($SRC_DIR, \%ignored_src, \%added_src);
my %check_src = list_dir($CHECK_DIR, undef, undef);
my %includes = list_dir($INCLUDE_DIR, \%ignored_include, \%added_include);

print_config_h(defines(@using_mods));
print_version_cpp($MAJOR_VERSION, $MINOR_VERSION, $PATCH_VERSION, @using_mods);
$includes{'config.h'} = $INCLUDE_DIR;
$lib_src{'version.cpp'} = $SRC_DIR;

mkdirs(($BUILD_DIR, catdir($BUILD_DIR, 'include'),
                    $BUILD_INC_DIR, $BUILD_SRC_DIR,
                    $BUILD_LIB_DIR, $BUILD_CHECK_DIR));
clean_out_stuff(($BUILD_INC_DIR, $BUILD_SRC_DIR));
copy_files($BUILD_DIR, $SRC_DIR, $INCLUDE_DIR, \%lib_src, \%includes);

%lib_src = map { $_ => $BUILD_SRC_DIR } keys %lib_src;
%includes = map { $_ => $BUILD_INC_DIR } keys %includes;

print_makefile($CC_BINARY_NAME{$cc} . $ccopts, $opt_flags, $mach_flags,
               $CC_LANG_FLAGS{$cc}, $CC_WARN_FLAGS{$cc},
               $CC_SO_OBJ_FLAGS{$cc}, $so_link_flags, $os,
               \%lib_src, \%check_src, \%includes, \%patched,
               using_libs(@using_mods));

exit;

sub process {
   my $l = $_[0];
   chomp($l);
   $l =~ s/#.*//;
   $l =~ s/^\s*//;
   $l =~ s/\s*$//;
   $l =~ s/\s\s*/ /;
   $l =~ s/\t/ /;
   $l;
}

sub check_for_file {
   my ($file,$mod) = @_;
   unless( -e $file ) { die
     "ERROR: Module $mod requires that file $file exist. This error\n",
     "should never occur; please contact the maintainers with details.\n";
   }
}

sub using_libs {
   my @using = @_;
   my %libs;
   foreach my $mod (@using) {
      foreach my $lib (sort keys %{ $MODULES{$mod}{'libs'} })
         { $libs{$lib} = 1; }
   }
   my @libarray;
   foreach (sort keys %libs) { push @libarray , $_; }
   return @libarray;
   }

sub defines {
   my @using = @_;
   my %defs;
   foreach (@using) {
      if(defined($MODULES{$_}{'define'})) {
         $defs{$MODULES{$_}{'define'}} = 1;
      }
   }
   my @defarray;
   foreach (sort keys %defs)
      { push @defarray , $_; }
   return @defarray;
   }

# Any other alternatives here?
sub portable_symlink {
   my ($from, $to) = @_;
   my $can_symlink = eval { symlink("",""); 1 };
   my $can_link = eval { link("",""); 1 };

   if($can_symlink)
      { symlink $from, $to or die "Can't symlink $from to $to ($!)"; }
   elsif($can_link)
      { link $from, $to or die "Can't link $from to $to ($!)"; }
   else { # Grrr... copy has different sematics than (sym)link
      # Does this work if the 'volume' thing is used?
      my(undef,$dirs,$file) = File::Spec->splitpath($from, 0);
      # Cut out empty entries and ..
      my $fixed_from = catfile((grep { /./ } grep { !/\.\./ }
                               File::Spec->splitdir($dirs)),
                               $file);
      copy ($fixed_from, $to) or die "Can't copy $fixed_from to $to ($!)";
      }
}

sub copy_files {
   my ($build_dir, $src_dir, $inc_dir, $lib_src, $includes) = @_;
   my %lib_src = %{ $lib_src };
   my %includes = %{ $includes };

   foreach (keys %lib_src) {
      portable_symlink(catfile(updir, updir, $lib_src{$_}, $_),
                       catfile($BUILD_SRC_DIR, $_));
   }

   foreach (keys %includes) {
      portable_symlink(catfile(updir, updir, updir, $includes{$_}, $_),
                       catfile($BUILD_INC_DIR, $_));
   }
}

sub list_dir {
  my ($dir, $check_hash, $merge_in) = @_;
  opendir DIR, $dir or die "Couldn't open directory $dir ($!)\n";
  my @list = grep { !/.dat$/ } grep { !/^\./ } readdir DIR;

  if(defined($check_hash)) {
     @list = grep { !exists($$check_hash{$_}) } @list;
  }

  close DIR;
  my %list = map { $_ => $dir } @list;

  if(defined($merge_in)) {
     my %merge_in = %{ $merge_in };
     foreach (keys %merge_in) {
        my $mod = $merge_in{$_};
        if(defined($list{$_})) {
           die "ERROR: Module $mod and the main source tree both have $_ ",
               "(this should never happen)\n";
        }
        $list{$_} = $mod;
     }
  }
  return %list;
}

sub clean_out_stuff {
   my (@dirs) = @_;
   foreach my $dir (@dirs) {
      my %files = list_dir($dir);
      foreach my $file (keys %files) {
         my $path = catfile($dir, $file);
         unlink $path or die "Could not unlink $path ($!)\n";
      }
   }    
}

sub mkdirs {
   my (@dirs) = @_;
   foreach my $dir (@dirs) {
      next if( -e $dir and -d $dir ); # skip it if it's already there
      mkdir($dir) or die "ERROR: Could not create directory $dir ($!)\n";
   }
}sub print_config_h
   {
   my (@defines) = @_;

   my $defines = "";
   if(@defines) { $defines .= "\n"; }
   foreach (sort @defines) {
      next if not defined $_ or not $_;
      $defines .= "#define OPENCL_EXT_$_\n"; }

   my $config_h = catfile ($INCLUDE_DIR, 'config.h');
   open CONFIG_H, ">$config_h" or die "Couldn't write $config_h ($!)\n";

   print CONFIG_H <<END_OF_CONFIG_H;
/*************************************************
* Configuration Header File                      *
* (C) 1999-2001 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_CONFIG_H__
#define OPENCL_CONFIG_H__

#include <opencl/types.h>

namespace OpenCL {

const u32bit DEFAULT_BUFFERSIZE = 1024;
$defines
}

#endif
END_OF_CONFIG_H
   }

sub print_version_cpp
   {
   my ($major, $minor, $patch, @mods) = @_;
   my $modstring = "";
   if(@mods) { $modstring = ' (modules: ' . join(" ", sort @mods) . ')'; }
   my $version_cpp = catfile ($SRC_DIR, 'version.cpp');
   open VERSION_CPP, ">$version_cpp" or
      die "Couldn't write $version_cpp ($!)\n";
   print VERSION_CPP <<END_OF_VERSION_CPP;
/*************************************************
* OpenCL Version Information                     *
* (C) 1999-2001 The OpenCL Project               *
*************************************************/

#include <opencl/opencl.h>

namespace OpenCL {

/*************************************************
* Return the version as a string                 *
*************************************************/
std::string version_string()
   {
   return "OpenCL v$major.$minor.$patch$modstring";
   }

/*************************************************
* Return parts of the version as numbers         *
*************************************************/
u32bit version_major() { return $major; }
u32bit version_minor() { return $minor; }
u32bit version_patch() { return $patch; }

}
END_OF_VERSION_CPP
   }

sub check_for_conflicts {
   my @mods = @_;
   my (%ignored, %added, %patched, %replaced, %defines);
   foreach my $mod (@mods) {
      if(defined ($MODULES{$mod}{'define'}))
         { conflicts($mod, $MODULES{$mod}{'define'}, 'define', \%defines); }

      sub check_hash {
         my ($mod, $do_what, $hash) = @_;
         foreach (keys %{ $MODULES{$mod}{$do_what} }) {
            conflicts($mod, $_, $do_what, $hash);
         }
      }

      check_hash($mod, 'replace', %replaced);
      check_hash($mod, 'add', %added);
      check_hash($mod, 'patch', %patched);
      check_hash($mod, 'ignore', %ignored);
   }
}

sub conflicts {
    my ($mod, $item, $do_what, $hashref) = @_;
    return if(!defined($item));
    if(defined($$hashref{$item})) {
       my $other_mod = $$hashref{$item};
       print "WARNING: Both $mod and $other_mod $do_what $item\n";
       }
    else { $$hashref{$item} = $mod }
}
sub get_modules_list
   {
   my $MOD_DIR = $_[0];
   my $dir = new DirHandle $MOD_DIR;
   if(!defined $dir) { return (); }

   my %MOD;

   while(defined($_ = $dir->read))
      {
      next if($_ eq '.' or $_ eq '..');
      my $MODULE = $_;
      my %modinfo = get_module_info($MODULE, $MOD_DIR);
      foreach (keys %modinfo) { $MOD{$MODULE}{$_} = $modinfo{$_}; }
      }
   return %MOD;
   }

sub get_module_info
   {
   my ($MODULE, $MOD_DIR) = @_;
   my %HASH;
   my $mod_dirname = catfile($MOD_DIR,$MODULE);
   my $mod_dir = new DirHandle $mod_dirname;
   if(!defined $mod_dir)
      { die "ERROR: Couldn't open dir $mod_dirname ($!)\n"; }

   my %MODFILES;
   my $have_config_file = 0;
   while(defined($_ = $mod_dir->read))
      {
      if($_ eq $MODULE) { $have_config_file = 1; }
      else { $MODFILES{$_} = undef; }
      }
   die "ERROR: Module $MODULE does not seem to have a description file\n"
      unless $have_config_file;

   my $desc_file = catfile($MOD_DIR,$MODULE,$MODULE);
   open MODFILE, "<$desc_file" or die
      "ERROR: Couldn't open file $desc_file, ($!)\n";

   $HASH{'needs_std'} = {};
   $HASH{'libs'} = {};

   $HASH{'add'} = {};
   $HASH{'replace'} = {};
   $HASH{'ignore'} = {};
   $HASH{'patch'} = {};

   while(<MODFILE>)
      {
      if($_ =~ /^realname \"(.*)\"/) { $HASH{'name'} = $1; }

      if($_ =~ /^patch_file ([\.\w]*) ([\.\w]*)/)
         { $HASH{'patch'}{$1} = $2; }
      if($_ =~ /^add_file ([\.\w]*)/) { $HASH{'add'}{$1} = undef; }
      if($_ =~ /^replace_file ([\.\w]*)/) { $HASH{'replace'}{$1} = undef; }
      if($_ =~ /^ignore_file ([\.\w]*)/) { $HASH{'ignore'}{$1} = undef; }

      if($_ =~ /^define (\w*)/) { $HASH{'define'} = $1; }
      if($_ =~ /^link_to (\w*)/) { $HASH{'libs'}{$1} = undef; }
      if($_ =~ /^subsystem (\w*)/) { $HASH{'subsystem'} = $1; }
      if($_ =~ /^needs_std (\w*)/) { $HASH{'needs_std'}{$1} = undef; }
      if($_ =~ /^uses_asm (\w*)/) { $HASH{'uses_asm'} = 1; }

      # Read in a list of supported CPU types (archs and/or submodels)
      if($_ =~ /^<arch>$/) {
         while(1) {
            $_ = process($_ = <MODFILE>);
            next unless $_;
            last if ($_ =~ m#^</arch>$#);
            $HASH{'arch'}{$_} = undef;
            }
         }

      # Read in a list of supported OSes
      if($_ =~ /^<os>$/) {
         while(1) {
            $_ = process($_ = <MODFILE>);
            next unless $_;
            last if ($_ =~ m#^</os>$#);
            $HASH{'os'}{$_} = undef;
            }
         }

      # Read in a list of supported compilers
      if($_ =~ /^<cc>$/) {
         while(1) {
            $_ = process($_ = <MODFILE>);
            next unless $_;
            last if ($_ =~ m#^</cc>$#);
            $HASH{'cc'}{$_} = undef;
            }
         }
      }
   return %HASH;
   }

sub load_module {
   my ($modname,$cc,$os,$arch,$sub,$no_asm,%module) = @_;

   # Check to see if everything is cool about system requirements
   if(defined($module{'os'}) and !exists($module{'os'}{$os}) and
         $os ne 'generic')
       { die "ERROR: Module '$modname' does not support $REALNAME{$os}\n"; }

   foreach (sort keys %{ $module{'needs_std'} }) {
       unless($os eq 'generic' or $OS_SUPPORTS_STD{$os}{$_}) {
          my $stdname = $_;
          if($REALNAME{$_}) { $stdname = $REALNAME{$_}; }
          die "ERROR: Module '$modname' needs $stdname, but $REALNAME{$os} ",
              "doesn't support it\n";
       }
    }

   if(defined($module{'arch'}) and $arch ne 'generic' and
      !exists($module{'arch'}{$arch}) and !exists($module{'arch'}{$sub}))
       { die "ERROR: Module '$modname' does not support ".
              "$REALNAME{$arch}/$sub\n"; }

   if(defined($module{'cc'}) and !exists($module{'cc'}{$cc}))
       { die "ERROR: Module '$modname' does not support $REALNAME{$cc}\n"; }

   if($no_asm and exists($module{'uses_asm'}))
       { print "WARNING: Module '$modname' disabled (uses assembly)\n"; 
         return; }

   handle_files($modname, $module{'replace'}, \&replace_file);
   handle_files($modname, $module{'ignore'},  \&ignore_file);
   handle_files($modname, $module{'add'},     \&add_file);
   handle_files($modname, $module{'patch'} ,  \&patch_file);
   }

sub handle_files {
   my($modname, $hash_scalar, $func) = @_;
   return unless defined($hash_scalar);
   my %hash = %{ $hash_scalar };
   foreach (sort keys %hash) {
      if(defined($hash{$_})) { &$func($modname, $_, $hash{$_}); }
      else                   { &$func($modname, $_); }
    }
}

sub full_path {
   my ($file,$modname) = @_;
   if(defined($modname)) { return catfile ($MOD_DIR, $modname, $file); }
   else {
      if($file =~ /\.h$/) { return catfile ($INCLUDE_DIR, $file); }
      elsif($file =~ /\.icc$/) { return catfile ($INCLUDE_DIR, $file); }
      elsif($file =~ /\.cpp$/) { return catfile ($SRC_DIR, $file); }
   }
}

sub patch_file {
   my ($modname,$file,$patch) = @_;
   ignore_file($modname, $file); # ignore the original
   $patch = full_path($patch, $modname);
   check_for_file($patch, $modname);

   my $patched_file = catfile ($MOD_DIR, $modname, $file);
   copy(full_path($file), $patched_file);

   if($OS_TYPE{$os} eq 'unix') {`patch -i $patch $patched_file`; }
   else                        { die "Don't know how to patch on $os\n"; }

   if($patched_file =~ /.cpp$/)
      { $added_src{$file} = catdir($MOD_DIR, $modname); }
   else { $added_include{$file} = catdir($MOD_DIR, $modname); }
   $patched{$file} = $modname;
   }

sub add_file {
   my ($modname,$file) = @_;
   check_for_file(full_path($file, $modname), $modname);
   if($file =~ /.cpp$/) { $added_src{$file} = catdir($MOD_DIR, $modname); }
   else                 { $added_include{$file} = catdir($MOD_DIR, $modname); }
}

sub ignore_file {
   my ($modname,$file) = @_;
   check_for_file(full_path($file), $modname);
   if($file =~ /.cpp$/) { $ignored_src{$file} = 1; }
   else                 { $ignored_include{$file} = 1; }
}

# This works because ignore file always runs on files in the main source tree,
# and add always works on the file in the modules directory.
sub replace_file {
   my ($modname,$file) = @_;
   ignore_file($modname, $file);
   add_file($modname, $file);
}

sub help
   {
   print <<ENDOFHELP;
Usage: $0 [--debug] [--disable-asm] [--modules=MODS] CC-OS-CPU

Options:
  --debug: tune compiler flags to allow debugging
  --disable-asm: disable all assembly (implied by --debug)
  --modules=MODS: add modules MODS to the library. MODS can be a single
     module or a comma separated list

You may use 'generic' for OS or CPU, which will disable any specializations
that would ordinarily be made (useful if your OS or CPU isn't listed).

CPU can be a generic family name or a specific model name. Common aliases
(for both families and models) are supported but not listed. Choosing a
specific submodel will usually result in code that will not run on earlier
versions of that architecture.

ENDOFHELP
   print_listing('CC', %CC_BINARY_NAME);
   print_listing('OS', %OS_SUPPORTS_ARCH);
   print_listing('CPU', %DEFAULT_SUBMODEL);
   if(%MODULES) { print_listing('MODS', %MODULES); }
   exit;
   }

sub print_listing
   {
   my ($header, %hash) = @_;
   print "$header: ";
   my $len = length "$header: ";
   foreach(sort(keys %hash))
      {
      if($len > 65) { print "\n   "; $len = 3; }
      print "$_ ";
      $len += length "$_ ";
      }
   print "\n";
   }

sub print_makefile {
   my($cc, $opt, $mach_opt, $lang, $warn, $so_obj, $so_link, $os,
      $lib_src, $check_src, $headers, $patched, @libs_used) = @_;

   my $make_shared = "no";
   if(($so_obj or $so_link) and $OS_SUPPORTS_SHARED{$os})
      { $make_shared = "yes"; }
   if($make_shared eq "no") { $so_obj = $so_link = ""; }

   open MAKEFILE, ">$MAKE_FILE" or die "Couldn't write $MAKE_FILE ($!)\n";

   my $os_type = $OS_TYPE{$os};
   if($os_type eq 'unix') {
       print_unix_makefile(\*MAKEFILE, $cc, $opt, $mach_opt, $lang, $warn,
                           $so_obj, $so_link, $make_shared, $os,
                           $lib_src, $check_src, $headers, $patched,
                           \@libs_used);
   }
   else {
      die "ERROR: This configure script does not know how to make ",
          "a makefile for makefile style $os_type";
   }
}

sub print_header {
   my ($fh, $string) = @_;
   print $fh "##################################################\n",
             "# $string", ' 'x(47-length($string)), "#\n",
             "##################################################\n";
}

# FIXME: A lot of this code can be shared by the other makefile writers
# (if they are ever written).
sub print_unix_makefile {
   my ($makefile, $cc, $opt, $mach_opt, $lang, $warn, $so_obj, $so_link, 
       $make_shared, $os, $src, $check, $header, $patched, $libs_used) = @_;

   my $__TAB__ = "\t";

   my %src = %{ $src };
   my %check = %{ $check };
   my %header = %{ $header };
   my %patched = %{ $patched };

   my $link_to = ""; foreach (@{ $libs_used }) { $link_to .= "-l$_ "; }
   chop($link_to) if($link_to);

   my $lib_flag = "\$(OPT_FLAGS) \$(MACH_FLAGS) \$(LANG_FLAGS) \$(WARN_FLAGS)";
   my $libs = "\$(STATIC_LIB)";
   if($make_shared eq "yes") { $lib_flag .= " \$(SO_OBJ_FLAGS)"; 
                               $libs .= " \$(SHARED_LIB)"; }

##################### COMMON CODE (PARTIALLY) ######################
   sub file_list {
      my ($put_in, $from, $to, %files) = @_;
      my $len = 20;
      my $list;
      foreach (sort keys %files) {
         my $file = $_;
         my $dir = $put_in;
         if(!defined($dir)) { $dir = $files{$_}; }
         if($len > 60) { $list .= "\\\n                "; $len = 20; }
         if(defined($from) and defined($to)) { $file =~ s/$from/$to/; }
         $list .= catfile ($dir, $file).' ';
         $len += length($file) + length($dir);
      }
      return $list;
   }

   my $headers = file_list(undef, undef, undef, %header);

   my $lib_src = file_list(undef, undef, undef, %src);
   my $lib_obj = file_list($BUILD_LIB_DIR, '.cpp', '.o', %src);
   my $check_src = file_list(undef, undef, undef, %check);
   my $check_obj = file_list($BUILD_CHECK_DIR, '.cpp', '.o', %check);

   my $remove_patched_files = file_list(undef, undef, undef,
                map { $_ => catdir($MOD_DIR, $patched{$_}) } keys %patched);
   if(defined($remove_patched_files))
      { $remove_patched_files = "$__TAB__\$(RM) $remove_patched_files\n"; }
   else { $remove_patched_files = ""; }

##################### / COMMON CODE (PARTIALLY) ######################

   print $makefile <<END_OF_MAKEFILE_HEADER;
##################################################
# Compiler Options                               #
##################################################
CXX           = $cc
OPT_FLAGS     = $opt
MACH_FLAGS    = $mach_opt
LANG_FLAGS    = $lang
WARN_FLAGS    = $warn
MAKE_SAHRED   = $make_shared
SO_OBJ_FLAGS  = $so_obj
SO_LINK_FLAGS = $so_link

##################################################
# Names and Version Numbers                      #
##################################################
NAME          = OpenCL
LC_NAME       = opencl

MAJOR         = $MAJOR_VERSION
MINOR         = $MINOR_VERSION
PATCH         = $PATCH_VERSION

VERSION       = \$(MAJOR).\$(MINOR).\$(PATCH)

##################################################
# Installation Settings                          #
##################################################
INSTALLROOT   = /usr/local

LIBDIR        = \$(INSTALLROOT)/lib
HEADERDIR     = \$(INSTALLROOT)/include/\$(LC_NAME)
DOCDIR        = \$(INSTALLROOT)/share/doc/\$(NAME)-\$(VERSION)

OWNER         = root
GROUP         = root
DATA_MODE     = 644
EXEC_MODE     = 755

##################################################
# Aliases for Common Programs                    #
##################################################
AR            = ar crs
CD            = \@cd
ECHO          = \@echo
INSTALL       = \@install -c -o \$(OWNER) -g \$(GROUP) -m
LN            = ln -fs
MKDIR         = \@mkdir
MKDIR_INSTALL = \@umask 022; mkdir -p -m \$(EXEC_MODE)
STRIP         = \@strip
RM            = \@rm -f
RM_R          = \@rm -rf

##################################################
# General Setup                                  #
##################################################
LIB_FLAGS     = $lib_flag
CHECK_FLAGS   = \$(OPT_FLAGS) \$(MACH_FLAGS) \$(LANG_FLAGS) \$(WARN_FLAGS)

CHECK         = check

# FIXME: Hardcoded
USER_DOCS     = readme.txt doc/license.txt doc/log.txt doc/thanks.txt \\
                doc/pgpkeys.asc
DEVEL_DOCS    = doc/doc.ps doc/relnotes.txt

HEADERS       = $headers

LIBSRCS       = $lib_src

CHECKSRCS     = $check_src

LIBOBJS       = $lib_obj

CHECKOBJS     = $check_obj

LIBRARIES     = $libs

LIBNAME       = lib\$(LC_NAME)
STATIC_LIB    = \$(LIBNAME).a
END_OF_MAKEFILE_HEADER

   if($make_shared eq 'yes') {
       print $makefile <<END_OF_SHARED_LIB_DECL;

SHARED_LIB    = \$(LIBNAME)-\$(MAJOR).\$(MINOR).\$(PATCH).so
SONAME        = \$(LIBNAME)-\$(MAJOR).so

SYMLINK0      = \$(LIBNAME)-\$(MAJOR).\$(MINOR).\$(PATCH).so
SYMLINK1      = \$(LIBNAME)-\$(MAJOR).\$(MINOR).so
SYMLINK2      = \$(LIBNAME)-\$(MAJOR).so
SYMLINK3      = \$(LIBNAME).so
SYMLINKS      = \$(SYMLINK1) \$(SYMLINK2) \$(SYMLINK3)

END_OF_SHARED_LIB_DECL
   }

   print MAKEFILE "all: \$(LIBRARIES)\n\n";
   print_header($makefile, "Build Commands");

   sub print_build_cmds {
      my ($fh, $dir, $flags, %files) = @_;
      foreach (sort keys %files) {
         my $src_file = catfile ($files{$_}, $_);
         my $obj_file = catfile ($dir, $_);
         $obj_file =~ s/.cpp/.o/;
         print $fh "$obj_file: $src_file\n",
            "\t\$(CXX) -Ibuild/include/ $flags -c \$? -o \$@\n\n";
      }
   }

   print_build_cmds($makefile, $BUILD_LIB_DIR, '$(LIB_FLAGS)', %src);
   print_build_cmds($makefile, $BUILD_CHECK_DIR, '$(CHECK_FLAGS)', %check);

   print_header($makefile, "Link Commands");

   print $makefile <<END_OF_LINK_COMMANDS;
\$(CHECK): \$(LIBRARIES) \$(CHECKOBJS)
$__TAB__\$(CXX) \$(CHECKOBJS) -L. $link_to -l\$(LC_NAME) -o \$(CHECK)
$__TAB__\$(STRIP) \$(CHECK)
$__TAB__\@./\$(CHECK) --validate

\$(STATIC_LIB): \$(LIBOBJS)
$__TAB__\$(RM) \$(STATIC_LIB)
$__TAB__\$(AR) \$(STATIC_LIB) \$(LIBOBJS)

END_OF_LINK_COMMANDS

   if($make_shared eq "yes") {
      print $makefile <<END_OF_SO_LINK_COMMAND;
\$(SHARED_LIB): \$(LIBOBJS)
$__TAB__\$(CXX) \$(SO_LINK_FLAGS) $link_to \$(LIBOBJS) -o \$(SHARED_LIB)
$__TAB__\$(LN) \$(SYMLINK0) \$(SYMLINK1)
$__TAB__\$(LN) \$(SYMLINK1) \$(SYMLINK2)
$__TAB__\$(LN) \$(SYMLINK2) \$(SYMLINK3)

END_OF_SO_LINK_COMMAND
    }

   print_header($makefile, "Misc Targets");

   print $makefile <<END_OF_MISC_STUFF;
static: \$(STATIC_LIB)

shared: \$(SHARED_LIB)

##################################################
# Fake Targets                                   #
##################################################
.PHONY = clean cleanconfig distclean install install_user install_devel

clean:
$__TAB__\$(RM_R) build/lib/* build/checks/*
$__TAB__\$(RM) \$(LIBRARIES) \$(SYMLINKS) \$(CHECK)

cleanconfig: clean
$__TAB__\$(RM) build/include/opencl/* build/src/*
$__TAB__\$(RM) Makefile include/config.h src/version.cpp

distclean: cleanconfig
$__TAB__\$(RM_R) build
$remove_patched_files
END_OF_MISC_STUFF

   print_header($makefile, 'Install Targets');

   my $install_shared = "";
   if($make_shared eq "yes") {
      $install_shared = 
        "$__TAB__\$(INSTALL) \$(EXEC_MODE) \$(SHARED_LIB) \$(LIBDIR)\n".
        "$__TAB__\$(CD) \$(LIBDIR); \$(LN) \$(SYMLINK0) \$(SYMLINK1)\n".
        "$__TAB__\$(CD) \$(LIBDIR); \$(LN) \$(SYMLINK1) \$(SYMLINK2)\n".
        "$__TAB__\$(CD) \$(LIBDIR); \$(LN) \$(SYMLINK2) \$(SYMLINK3)";
   }

   print $makefile <<END_OF_INSTALL_SCRIPTS;
install: \$(LIBRARIES)
$__TAB__\$(ECHO) "Installing \$(NAME) into \$(INSTALLROOT)... "
$__TAB__\$(MKDIR_INSTALL) \$(DOCDIR)
$__TAB__\$(MKDIR_INSTALL) \$(HEADERDIR)
$__TAB__\$(MKDIR_INSTALL) \$(LIBDIR)
$__TAB__\$(INSTALL) \$(DATA_MODE) \$(USER_DOCS) \$(DOCDIR)
$__TAB__\$(INSTALL) \$(DATA_MODE) \$(DEVEL_DOCS) \$(DOCDIR)
$__TAB__\$(INSTALL) \$(DATA_MODE) \$(HEADERS) \$(HEADERDIR)
$__TAB__\$(INSTALL) \$(DATA_MODE) \$(STATIC_LIB) \$(LIBDIR)
$install_shared

install_user: \$(LIBRARIES)
$__TAB__\$(MKDIR_INSTALL) \$(DOCDIR)
$__TAB__\$(MKDIR_INSTALL) \$(LIBDIR)
$__TAB__\$(INSTALL) \$(DATA_MODE) \$(USER_DOCS) \$(DOCDIR)
$install_shared

install_devel: \$(LIBRARIES)
$__TAB__\$(MKDIR_INSTALL) \$(DOCDIR)
$__TAB__\$(MKDIR_INSTALL) \$(HEADERDIR)
$__TAB__\$(MKDIR_INSTALL) \$(LIBDIR)
$__TAB__\$(INSTALL) \$(DATA_MODE) \$(DEVEL_DOCS) \$(DOCDIR)
$__TAB__\$(INSTALL) \$(DATA_MODE) \$(HEADERS) \$(HEADERDIR)
$__TAB__\$(INSTALL) \$(DATA_MODE) \$(STATIC_LIB) \$(LIBDIR)

END_OF_INSTALL_SCRIPTS

}
